@@ -31,9 +31,7 @@ import six
 
 import sge
 from sge import r
-from sge.r import (o_update_collision_lists, o_get_image_width,
-                   o_get_normal_image_width, o_get_image_height,
-                   o_get_normal_image_height, o_get_origin_offset, o_set_speed,
+from sge.r import (o_update_collision_lists, o_get_origin_offset, o_set_speed,
                    s_get_image, s_get_precise_mask)
 
 
@@ -669,21 +667,24 @@ class Object(object):
     @property
     def mask_x(self):
         if self.collision_precise:
-            origin_x = self.image_origin_x
-            id_ = (self.sprite.width, self.sprite.height, origin_x,
-                   self.image_origin_y, self.image_xscale, self.image_yscale,
-                   self.image_rotation)
-            if id_ in self.__masks_x_offset:
-                offset = self.__masks_x_offset[id_]
-                return self.x - (origin_x + offset)
-            elif self.image_rotation % 180:
-                width = o_get_image_width(self)
-                normal_width = o_get_normal_image_width(self)
-                offset = (width - normal_width) / 2
-                self.__masks_x_offset[id_] = offset
+            if self.image_rotation % 180:
+                origin_x = self.image_origin_x
+                i = ("o_mask_x_offset", self, self.sprite, origin_x,
+                     self.image_origin_y, self.image_xscale, self.image_yscale,
+                     self.image_rotation)
+                offset = r.cache.get(i)
+                if offset is None:
+                    width = s_get_image(self.sprite, self.image_index,
+                                        self.image_xscale, self.image_yscale,
+                                        self.image_rotation).get_width()
+                    normal_width = s_get_image(self.sprite, self.image_index,
+                                               self.image_xscale,
+                                               self.image_yscale).get_width()
+                    offset = (width - normal_width) / 2
+
+                r.cache.add(i, offset)
                 return self.x - (origin_x + offset)
             else:
-                self.__masks_x_offset[id_] = 0
                 return self.x - origin_x
         else:
             return self.bbox_left
@@ -691,21 +692,24 @@ class Object(object):
     @property
     def mask_y(self):
         if self.collision_precise:
-            origin_y = self.image_origin_y
-            id_ = (self.sprite.width, self.sprite.height, self.image_origin_x,
-                   origin_y, self.image_xscale, self.image_yscale,
-                   self.image_rotation)
-            if id_ in self.__masks_y_offset:
-                offset = self.__masks_y_offset[id_]
-                return self.y - (origin_y + offset)
-            elif self.image_rotation % 180:
-                height = o_get_image_height(self)
-                normal_height = o_get_normal_image_height(self)
-                offset = (height - normal_height) / 2
-                self.__masks_y_offset[id_] = offset
+            if self.image_rotation % 180:
+                origin_y = self.image_origin_y
+                i = ("o_mask_y_offset", self, self.sprite, self.image_origin_x,
+                     origin_y, self.image_xscale, self.image_yscale,
+                     self.image_rotation)
+                offset = r.cache.get(i)
+                if offset is None:
+                    height = s_get_image(self.sprite, self.image_index,
+                                         self.image_xscale, self.image_yscale,
+                                         self.image_rotation).get_height()
+                    normal_height = s_get_image(self.sprite, self.image_index,
+                                                self.image_xscale,
+                                                self.image_yscale).get_height()
+                    offset = (height - normal_height) / 2
+
+                r.cache.add(i, offset)
                 return self.y - (origin_y + offset)
             else:
-                self.__masks_y_offset[id_] = 0
                 return self.y - origin_y
         else:
             return self.bbox_top
@@ -761,8 +765,6 @@ class Object(object):
         self.rd["collision_areas"] = []
         self.rd["colliders"] = []
         self.__masks = {}
-        self.__masks_x_offset = {}
-        self.__masks_y_offset = {}
 
         self.rd["sprite"] = sprite
         if sprite is not None:
